ColorConfiguratorListDialog = {}

local ColorConfiguratorListDialog_mt = Class(ColorConfiguratorListDialog, MessageDialog)

ColorConfiguratorListDialog.CONTROLS = {
	"list"
}

function ColorConfiguratorListDialog.new(target, custom_mt, colorConfigurator)
	local self = MessageDialog.new(target, custom_mt or ColorConfiguratorListDialog_mt)

	self:registerControls(ColorConfiguratorListDialog.CONTROLS)

	self.colorConfigurator = colorConfigurator

	self.disableOpenSound = true
	self.brandColors = {}

	return self
end

function ColorConfiguratorListDialog:onMapLoad()
	for name, color in pairs(g_brandColorManager.brandColors) do
		local r, g, b, _ = unpack(color)

		table.insert(self.brandColors, {name = string.gsub(name, "_", " "), color = {r, g, b}})
	end

	table.sort(self.brandColors, function (a, b)
		return a.name < b.name
	end)

	self.list:reloadData()
end

function ColorConfiguratorListDialog:onClickColor(list, section, index)
	local section = self.list.selectedSectionIndex
	local index = self.list.selectedIndex

	if self:getNumberOfSections() == 1 or section == 2 then
		self.colorConfigurator.dialog:setValues(self.brandColors[index].color)
	else
		local baseConfig = self.colorConfigurator.baseConfigs[index]
		local color = baseConfig.color
		local mat = baseConfig.material

		self.colorConfigurator.dialog:setValues({color[1], color[2], color[3], mat}, true)
	end

	self:playSample(GuiSoundPlayer.SOUND_SAMPLES.CLICK)
	self:close()
end

function ColorConfiguratorListDialog:populateCellForItemInSection(list, section, index, cell)
	local name = ""
	local r, g, b = 0, 0, 0
	local isMetallic = false
	local isButtonVisible = false

	if self:getNumberOfSections() == 1 or section == 2 then
		local brandColor = self.brandColors[index]

		name = brandColor.name
		r, g, b = unpack(brandColor.color)
	else
		local baseColor = self.colorConfigurator.baseConfigs[index]

		name = baseColor.name
		r, g, b = unpack(baseColor.color)
		isMetallic = ConfigurationUtil.isColorMetallic(baseColor.material)
		isButtonVisible = true
	end

	cell:getAttribute("title"):setText(name)
	cell:getAttribute("image"):setImageColor(nil, r, g, b, 1)
	cell:getAttribute("image"):setVisible(not isMetallic)
	cell:getAttribute("imageMetallic"):setImageColor(nil, r, g, b, 1)
	cell:getAttribute("imageMetallic"):setVisible(isMetallic)
	cell:getAttribute("button"):setVisible(isButtonVisible)
end

function ColorConfiguratorListDialog:getNumberOfItemsInSection(list, section)
	if self:getNumberOfSections() == 1 or section == 2 then
		return #self.brandColors
	end

	return #self.colorConfigurator.baseConfigs
end

function ColorConfiguratorListDialog:getNumberOfSections()
	return #self.colorConfigurator.baseConfigs == 0 and 1 or 2
end

function ColorConfiguratorListDialog:getTitleForSectionHeader(list, section)
	if self:getNumberOfSections() == 1 or section == 2 then
		return g_i18n:getText("colorConfigurator_ui_gameColors")
	end

	return g_i18n:getText("colorConfigurator_ui_userColors")
end

function ColorConfiguratorListDialog:onClickRemoveColor(element)
	local listItem = element.parent.parent
	local index = listItem.indexInSection
	local baseConfigs = self.colorConfigurator.baseConfigs
	local baseConfig = baseConfigs[index]

	g_gui:showYesNoDialog({
		text = string.format(g_i18n:getText("colorConfigurator_ui_removeColor"), baseConfig.name),
		callback = function (yes)
			if yes then
				self.colorConfigurator:removeBaseConfig(baseConfig)
				self.list:reloadData()
			end
		end
	})
end

function ColorConfiguratorListDialog:keyEvent(unicode, sym, modifier, isDown, eventUsed)
	if ColorConfiguratorListDialog:superClass().keyEvent(self, unicode, sym, modifier, isDown, eventUsed) then
		return true
	end

	if isDown and unicode ~= 0 and unicode ~= 10 and unicode ~= 13 then
		local text = utf8ToUpper(unicodeToUtf8(unicode))

		if not text:find("[^%w_.]") then
			local index = nil

			for i, color in pairs(self.brandColors) do
				if text == utf8ToUpper(color.name:sub(1, 1)) then
					index = i

					break
				end
			end

			if index ~= nil then
				self.list:setSelectedItem(self:getNumberOfSections(), index)

				return true
			end
		end
	end

	return false
end